import React, { useState, useEffect } from 'react'
import { useAntdTable, useRequest } from 'ahooks'
import { sha256 } from 'js-sha256'
import { Table, Input, Form, Button, Row, Modal, Result, message } from 'antd'
import { FormItemProps } from 'antd/lib/form'
import { ColumnProps } from 'antd/lib/table'
import { Helmet } from 'react-helmet'
import { pageProps } from '../../router/config'
import Witherror from '../../commons/witherror'
const createformProps: (type: 'update' | 'add') => FormItemProps[] = (type) => {
  return [
    {
      name: 'username',
      label: '用户名',
      children: <Input placeholder='请输入用户名!'></Input>,
      required: true,
      rules: [
        {
          required: true,
          message: '请输入用户名',
        },
      ],
    },
    {
      name: 'password',
      label: '密码',
      children: <Input type='password' placeholder='请输入密码!'></Input>,
      required: type === 'add',
      rules: [
        {
          required: type === 'add',
          message: '请输入密码',
        },
      ],
    },
    {
      name: 'confirm_password',
      label: '确认密码',
      children: <Input type='password' placeholder='请输重复输入密码!'></Input>,
      required: type === 'add',
      dependencies: ['password'],
      rules: [
        {
          required: type === 'add',
          message: '请输入密码',
        },
        ({ getFieldValue }) => ({
          validator(_, value) {
            if (!value || getFieldValue('password') === value) {
              return Promise.resolve()
            }
            return Promise.reject(new Error('密码不一致!'))
          },
        }),
      ],
    },
    {
      name: 'phone',
      label: '手机号',
      required: false,
      children: <Input maxLength={11} minLength={11} placeholder='请输入手机号'></Input>,
      rules: [
        {
          type: 'string',
          len: 11,
          pattern: /^1[34578][0-9]{9}$/,
          message: '请输入正确的手机号',
        },
      ],
    },
    {
      name: 'email',
      label: '邮箱',
      rules: [
        {
          type: 'email',
          message: '请输入正确的邮箱',
        },
      ],
      required: false,
      children: <Input placeholder='请输入邮箱'></Input>,
    },
  ]
}

interface PersonProps {
  username: string
  phone: string
  email: string
  id: number
  created_at: string
}

/**
 * @description 管理员列表
 * @param props
 * @returns
 */
type Iprops = pageProps<{ id: string }>
const adminperson: React.FC<Iprops> = ({ title, match: { params: routeParams } }) => {
  const [form] = Form.useForm()
  const [insertState, changeInsertState] = useState({ visible: false })
  const [editState, changeEditState] = useState<{ visible: boolean; values?: PersonProps }>({
    visible: false,
  })
  const {
    tableProps,
    error: getListError,
    params: getListParams,
    pagination,
    run: requestList,
  } = useAntdTable((params) => {
    return {
      url: '/v1/organization/user_list',
      params: {
        organization_code: routeParams.id,
        page: params.current,
      },
    }
  }, {})
  const { run: requestAdd, loading: addLoading } = useRequest(
    (data) => {
      return {
        url: '/v1/organization/user',
        method: 'post',
        data: {
          organization_code: routeParams.id,
          ...data,
        },
      }
    },
    {
      manual: true,
      onSuccess() {
        message.success('添加成功')
        requestList({ ...getListParams, ...pagination })
        form.resetFields()
        changeInsertState({ visible: false })
      },
      onError(error) {
        message.error(error.message)
      },
    }
  )
  const { run: requestUpdate, loading: updateLoading } = useRequest(
    (data) => {
      return {
        url: '/v1/organization/user',
        method: 'put',
        data: {
          organization_code: routeParams.id,
          ...data,
        },
      }
    },
    {
      manual: true,
      onSuccess() {
        message.success('更新成功')
        requestList({ ...getListParams, ...pagination })
        form.resetFields()
        changeEditState({ visible: false })
      },
      onError(error) {
        message.error(error.message)
      },
    }
  )
  useEffect(() => {
    if (editState.values) {
      form.setFieldsValue(editState.values)
    }
  }, [editState.values])
  /**
   * @description 新增提交
   */
  const handleAddSubmit = () => {
    form.validateFields().then((data) => {
      requestAdd({ ...data, password: sha256(data.password) })
    })
  }
  /**
   * @description 更新提交
   */
  const handleUpdateSubmit = () => {
    form.validateFields().then((data) => {
      const requestData = { ...editState.values, ...data }
      if (data.password) {
        Object.assign(requestData, { ...data, password: sha256(data.password) })
      }
      requestUpdate(requestData)
    })
  }
  const colums: ColumnProps<PersonProps>[] = [
    {
      title: '账号名',
      dataIndex: 'username',
    },
    {
      title: '手机号',
      dataIndex: 'phone',
    },
    {
      title: '邮箱',
      dataIndex: 'email',
    },
    {
      title: '创建时间',
      dataIndex: 'created_at',
    },
    {
      title: '操作',
      render(record) {
        return (
          <Button
            onClick={() => {
              changeEditState({ visible: true, values: record })
            }}
          >
            编辑
          </Button>
        )
      },
    },
  ]

  return (
    <Witherror
      error={getListError}
      onRetry={() => {
        requestList({ ...getListParams, ...pagination })
      }}
    >
      <Helmet>
        <title>{title}</title>
      </Helmet>
      <Row style={{ margin: '20px 0' }} justify='end'>
        <Button
          type='primary'
          onClick={() => {
            changeInsertState((preState) => ({ ...preState, visible: true }))
          }}
        >
          新建
        </Button>
      </Row>
      <Table
        rowKey={(record) => record.id}
        columns={colums}
        {...tableProps}
        pagination={{ hideOnSinglePage: true }}
      ></Table>
      <Modal
        okButtonProps={{
          loading: addLoading,
        }}
        onOk={handleAddSubmit}
        maskClosable={false}
        visible={insertState.visible}
        title='新建管理员'
        onCancel={() => {
          form.resetFields()
          changeInsertState((preState) => ({ ...preState, visible: false }))
        }}
      >
        <Form labelCol={{ span: 6 }} labelAlign='right' form={form}>
          {createformProps('add').map((element) => {
            return <Form.Item {...element} key={element.name as string}></Form.Item>
          })}
        </Form>
      </Modal>
      {editState.values && Object.keys(editState.values).length ? (
        <Modal
          okButtonProps={{
            loading: updateLoading,
          }}
          onOk={handleUpdateSubmit}
          maskClosable={false}
          visible={editState.visible}
          title='编辑管理员'
          onCancel={() => {
            form.resetFields()
            changeEditState(() => ({ visible: false }))
          }}
        >
          <Form
            labelCol={{ span: 6 }}
            labelAlign='right'
            form={form}
            initialValues={editState.values}
          >
            {createformProps('update').map((element) => {
              return <Form.Item {...element} key={element.name as string}></Form.Item>
            })}
          </Form>
        </Modal>
      ) : null}
    </Witherror>
  )
}
adminperson.displayName = 'adminperson'
export default adminperson
