import React, { useMemo, useState, useEffect } from 'react'
import {
  Table,
  Space,
  Button,
  Input,
  Row,
  Col,
  Form,
  InputNumber,
  Switch,
  Modal,
  message,
  Typography,
} from 'antd'
import { useRequest } from 'ahooks'

import { withRouter, RouteComponentProps } from 'react-router-dom'
import { ColumnProps } from 'antd/lib/table'
import WithErrorPage from '../../commons/witherror'
const { Search } = Input
const { Text } = Typography

interface organizationProps {
  organization_code: string
  organization_name: string
  end_time: number
  is_expire: boolean
  is_disable: boolean
}

const colums: ColumnProps<organizationProps>[] = [
  {
    title: '账号ID',
    dataIndex: 'organization_code',
    key: 'organization_code',
  },
  {
    title: '账号名称',
    dataIndex: 'organization_name',
    key: 'organization_name',
  },
  {
    title: '剩余有效期(天)',
    dataIndex: 'end_time',
    key: 'end_time',
  },
  {
    title: '认证key',
    key: 'key',
    dataIndex: 'key',
  },
  {
    title: '过期状态',
    dataIndex: 'is_expire',
    key: 'is_expire',
    render(dataKey) {
      return dataKey ? <Text type='warning'>已过期</Text> : <Text type='success'>未过期</Text>
    },
  },
  {
    title: '启用状态',
    dataIndex: 'is_disable',
    key: 'is_disable',
    render(dataKey) {
      return dataKey ? <Text type='warning'>已停用</Text> : <Text type='success'>已启用</Text>
    },
  },
]

/**
 * @description 机构列表页面
 * @param param0
 * @returns
 */
const mechanism: React.FC<RouteComponentProps> = ({ history }) => {
  const [form] = Form.useForm()
  const [{ filter, page }, changeSearchState] = useState({
    filter: '',
    page: 1,
  })
  const [insertModalState, changeInsertModalState] = useState({ visible: false })
  const [editModalState, changeEditModalState] = useState<{
    visible: boolean
    values?: organizationProps
  }>({ visible: false, values: undefined })
  useEffect(() => {
    if (editModalState.values) {
      form.setFieldsValue(editModalState.values)
    }
  }, [editModalState.values])
  const {
    error: requestListError,
    run,
    params,
    data: listData = { page: 1, total: 0, list: [] },
  } = useRequest(
    (params) => {
      return {
        url: '/v1/organization/list',
        params: {
          page_size: 10,
          ...params,
        },
      }
    },
    {
      refreshDeps: [filter, page],
      cacheKey: 'mechanismlist',
      staleTime: 30000,
    }
  )
  const { run: runRequestInsert, loading: requestInsertLoading } = useRequest(
    (data) => {
      return {
        url: '/v1/organization',
        method: 'post',
        data,
      }
    },
    {
      manual: true,
      onError(error) {
        message.error(error.message)
      },
      onSuccess() {
        message.success('新增机构成功!')
        changeInsertModalState({ visible: false })
        run({ filter, page })
      },
    }
  )
  const { run: runRequestUpdate, loading: updateLoading } = useRequest(
    (data) => {
      return {
        url: '/v1/organization',
        method: 'put',
        data,
      }
    },
    {
      manual: true,
      onError(error) {
        message.error(error.message)
      },
      onSuccess() {
        message.success('更新机构成功!')
        changeEditModalState({ visible: false })
        run({ filter, page })
      },
    }
  )
  /**
   * @description 新增提交
   */
  const handleSubmitAdd = () => {
    form.validateFields().then((data) => {
      runRequestInsert({ ...data, is_disable: Boolean(data.is_disable) })
    })
  }
  /**
   * @description 更新提交
   */
  const handleSubmitUpdate = () => {
    form.validateFields().then((data) => {
      runRequestUpdate({
        ...data,
        is_disable: Boolean(data.is_disable),
        organization_code: editModalState.values?.organization_code,
      })
    })
  }
  const columsWithOpertion = useMemo(() => {
    return colums.concat({
      title: '操作',
      render(record) {
        return (
          <Space>
            <Button
              onClick={() => {
                changeEditModalState((preState) => ({ ...preState, visible: true, values: record }))
              }}
            >
              编辑
            </Button>
            <Button
              onClick={() => {
                history.push(`/mechanism/${record.organization_code}`)
              }}
            >
              管理员
            </Button>
          </Space>
        )
      },
    })
  }, [])
  const InsertForms = (
    <Form labelCol={{ span: 5 }} form={form}>
      <Form.Item
        name='organization_name'
        label='账号名称'
        required
        rules={[
          {
            required: true,
            message: '请输入账号名称',
          },
        ]}
      >
        <Input placeholder='请输入账号名称'></Input>
      </Form.Item>
      <Form.Item
        name='month'
        label='有效期(增)'
        required
        rules={[
          {
            required: true,
            message: '请设置有效期',
          },
        ]}
      >
        <InputNumber
          min={0}
          formatter={(value) => `${value} 月`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}
          placeholder='请设置有效期'
          style={{ width: '100%' }}
        ></InputNumber>
      </Form.Item>
      <Form.Item
        normalize={(value) => Boolean(value)}
        name='is_disable'
        label='停用'
        valuePropName='checked'
      >
        <Switch></Switch>
      </Form.Item>
    </Form>
  )
  return (
    <WithErrorPage error={requestListError} onRetry={() => run(params)}>
      <Row style={{ margin: '20px 0' }} justify='space-between'>
        <Col span={4}>
          <Search
            placeholder='请输入用户名'
            onSearch={(filter) => {
              changeSearchState((preState) => ({ ...preState, filter }))
            }}
          ></Search>
        </Col>
        <Col>
          <Button
            type='primary'
            onClick={() => {
              changeInsertModalState((preState) => ({ ...preState, visible: true }))
            }}
          >
            新建
          </Button>
        </Col>
      </Row>
      <Table
        dataSource={listData.list}
        rowKey={(row) => row.organization_code}
        columns={columsWithOpertion}
        pagination={{
          hideOnSinglePage: true,
          total: listData.total,
          current: listData.page,
          onChange(page) {
            changeSearchState((preState) => ({ ...preState, page }))
          },
        }}
      ></Table>
      {/* 新增窗口 */}
      <Modal
        maskClosable={false}
        title='新建机构'
        okButtonProps={{
          loading: requestInsertLoading,
        }}
        destroyOnClose
        visible={insertModalState.visible}
        onOk={handleSubmitAdd}
        onCancel={() => {
          form.resetFields()
          changeInsertModalState((preState) => ({ ...preState, visible: false }))
        }}
      >
        {InsertForms}
      </Modal>
      {/* 编辑窗口 */}
      {editModalState.values && Object.keys(editModalState.values).length ? (
        <Modal
          maskClosable={false}
          title='编辑机构'
          okButtonProps={{
            loading: updateLoading,
          }}
          destroyOnClose
          visible={editModalState.visible}
          onCancel={() => {
            form.resetFields()
            changeEditModalState(() => ({ visible: false }))
          }}
          onOk={handleSubmitUpdate}
        >
          {InsertForms}
        </Modal>
      ) : null}
    </WithErrorPage>
  )
}
mechanism.displayName = 'mechanism'
export default withRouter(mechanism)
